from django.contrib import admin
from enumfields.admin import EnumFieldListFilter

from arkindex.ponos.models import Artifact, Task


class ArtifactInline(admin.TabularInline):
    model = Artifact
    fields = ("id", "path", "size", "content_type", "created")
    readonly_fields = fields
    # Artifacts cannot be added
    extra = 0

    def has_change_permission(self, request, obj=None):
        """
        Artifacts cannot be modified since all fields are read-only.
        We have to make this explicit to Django. Otherwise, it tries to update all the artifacts
        without receiving any field from the form when saving, so all the required fields are missing
        and users get "Please correct the error below:" with no actual error being displayed.
        """
        return False


class TaskAdmin(admin.ModelAdmin):
    list_display = (
        "id",
        "slug",
        "run",
        "state",
        "process_id",
        "updated",
        "priority",
    )
    list_filter = [("state", EnumFieldListFilter)]
    inlines = [
        ArtifactInline,
    ]
    readonly_fields = (
        "id",
        "created",
        "updated",
        "container",
        "shm_size",
        "original_task",
    )
    fieldsets = (
        (
            None,
            {
                "fields": (
                    "id",
                    "slug",
                    "run",
                    "depth",
                    "state",
                    "process",
                    "priority",
                    "original_task",
                ),
            },
        ),
        ("Dates", {"fields": ("created", "updated", "expiry")}),
        (
            "Docker",
            {
                "fields": (
                    "image",
                    "command",
                    "env",
                    "container",
                    "shm_size",
                    "extra_files",
                )
            },
        ),
    )
    raw_id_fields = ("process", )


class TaskInline(admin.TabularInline):
    model = Task
    fields = ("id", "slug", "run", "depth", "state", "container")
    readonly_fields = ("slug", "run", "depth", "container")
    extra = 0


admin.site.register(Task, TaskAdmin)
